/*
Magnetostatic problem (scalar magnetic potential), based on t5.pro from FAQ
*/

Include "par.inc";  // defect parameters

Group {
  Steel    = Region[ Steel_Code ];
  Air      = Region[ Air_Code ];
  Magnet   = Region[ Magnet_Code ];
  NonSteel = Region[ {Air, Magnet} ];
  Domain   = Region[ {NonSteel, Steel} ];
  Surface0 = Region[ Surface0_Code ];      // symetry and boundary
}

  Include "mu_steel_puc_couples.inc";  // steel curve

Function {
  mu0 = 4.e-7 * Pi ;
  muMagnet = 1.1 * mu0;
  mu[ Air ] = mu0;
  mu[ Magnet ] = muMagnet;
  hc[ Magnet ] = Vector[hcMag, 0., 0.];

  fmusteel[]=InterpolationLinear[$1]{List[mu_steel_couples]};
  mu [ Steel ] = fmusteel[Norm[$1]];

  // steel parameters for mu[Steel] = 1./(as + bs * Norm[$1]);
  //as = 471.384;
  //bs = 0.452111;
  //mu[Steel] = 1./(as + bs * Norm[$1]);

  // steel parameters for mu[Steel] = 1./(as + bs * Norm[$1] ^ cs);
  //as = 196.57;
  //bs = 0.74843;
  //cs = 0.963545;
  // mu[ Steel ] = 1./(as + bs * Norm[$1] ^ cs);
  // mu[ Steel ] = Norm[$1] < 830 ? 0.0014646258 : 1./(as + bs * Norm[$1] ^ cs);
}

Constraint {

  { Name phi ;
    Case {
      { Region Surface0 ; Value 0. ; }
    }
  }
}

Jacobian {
  { Name MyJac;
    Case {
      { Region All; Jacobian Vol; }
    }
  }
}

Integration {
  { Name MyInt;
    Case {
      { Type Gauss;
        Case {
          { GeoElement Tetrahedron; NumberOfPoints 5; }
          { GeoElement Triangle; NumberOfPoints 3; }
        }
      }
    }
  }
}

FunctionSpace {
  { Name Hgrad_phi; Type Form0;
    BasisFunction {
      { Name sn; NameOfCoef phin; Function BF_Node;
        Support Domain; Entity NodesOf[ All ]; }
      // uncomment the following for 2nd order interpolation:
      //{ Name sn2; NameOfCoef phin2; Function BF_Node_2E;
      //  Support Domain; Entity EdgesOf[ All ]; }
    }
    Constraint {
      { NameOfCoef phin ; EntityType NodesOf ; NameOfConstraint phi ; }
    }
  }
}

Formulation {
  { Name MagSta_phi; Type FemEquation;
    Quantity {
      { Name phi; Type Local; NameOfSpace Hgrad_phi; }
    }
    Equation {
      Galerkin { [ - mu[{d phi}] * Dof{d phi} , {d phi} ];
                 In Domain; Jacobian MyJac; Integration MyInt; }
      Galerkin { [ - muMagnet * hc[] , {d phi} ];
                 In Magnet; Jacobian MyJac; Integration MyInt; }
    }
  }
}

// Resolution parameters
NL_NbrMaxIter = 100;
NL_Eps = 1.e-4;
NL_Relax = 1;

Resolution {
  { Name MagSta_phi;
    System {
      { Name A; NameOfFormulation MagSta_phi; }
    }
    Operation {
      IterativeLoop[NL_NbrMaxIter, NL_Eps, NL_Relax] {
        GenerateJac[A]; SolveJac[A];
      }
      SaveSolution[A];
    }
  }
}

PostProcessing {
  { Name MagSta_phi; NameOfFormulation MagSta_phi;
    Quantity {
      { Name phi; Value { Local { [ {phi} ]; In Domain; Jacobian MyJac;} } }
      { Name h;   Value { Local { [ {d phi} ]; In Domain; Jacobian MyJac;} } }
      { Name b;   Value
           {
               Local { [ - mu[{d phi}] * {d phi} ]; In Domain; Jacobian MyJac; }
               Local { [ - muMagnet * hc[] ]    ; In Magnet ; Jacobian MyJac ; }
           }
      }
    }
  }
}

PostOperation {
  { Name phi; NameOfPostProcessing MagSta_phi;
    Operation {
      // Basic values
      Print[ phi, OnElementsOf Domain, File "phi.pos"] ;
      Print[ b,   OnElementsOf Domain, File "b.pos", Depth 0 ] ;
      Print[ h,   OnElementsOf Domain, File "h.pos", Depth 0 ] ;
    }
  }

  { Name grid; NameOfPostProcessing MagSta_phi;
    Operation {
      // Bidimensional grid for internal defect for gnuplot
      Print[ b, OnGrid {$A,$B,t/2+0.004}{0.0001:AcoX-0.03:0.001, 0.0001:AcoY-0.001:0.001, 0} , Format Gnuplot,
        File "bxyz0040i.txt" ];
      Print[ b, OnGrid {$A,$B,t/2+0.004}{0.0001:AcoX-0.03:0.001, 0.0001:AcoY-0.001:0.001, 0} , Format Gnuplot,
        File > "bxyz0040i.txt", ChangeOfCoordinates {-$X,$Y,$Z}, ChangeOfValues {2*$Val0/2}];
      Print[ b, OnGrid {$A,$B,t/2+0.004}{0.0001:AcoX-0.03:0.001, 0.0001:AcoY-0.001:0.001, 0} , Format Gnuplot,
        File > "bxyz0040i.txt", ChangeOfCoordinates {$X,-$Y,$Z}, ChangeOfValues {2*$Val0/2}];
      Print[ b, OnGrid {$A,$B,t/2+0.004}{0.0001:AcoX-0.03:0.001, 0.0001:AcoY-0.001:0.001, 0} , Format Gnuplot,
        File > "bxyz0040i.txt", ChangeOfCoordinates {-$X,-$Y,$Z}, ChangeOfValues {2*$Val0/2}];

      // idem for gmsh
      Print[ b, OnGrid {$A,$B,t/2+0.004}{0.0001:AcoX-0.03:0.002, 0.0001:AcoY-0.001:0.001, 0} ,
        File "bxyz0040i.pos" ];

      // Bidimensional grid for external defect for gnuplot
      Print[ b, OnGrid {$A,$B,-t/2-0.004}{0.0001:AcoX-0.03:0.001, 0.0001:AcoY-0.001:0.001, 0} , Format Gnuplot,
        File "bxyz0040e.txt" ];
      Print[ b, OnGrid {$A,$B,-t/2-0.004}{0.0001:AcoX-0.03:0.001, 0.0001:AcoY-0.001:0.001, 0} , Format Gnuplot,
        File > "bxyz0040e.txt", ChangeOfCoordinates {-$X,$Y,$Z}, ChangeOfValues {2*$Val0/2}];
      Print[ b, OnGrid {$A,$B,-t/2-0.004}{0.0001:AcoX-0.03:0.001, 0.0001:AcoY-0.001:0.001, 0} , Format Gnuplot,
        File > "bxyz0040e.txt", ChangeOfCoordinates {$X,-$Y,$Z}, ChangeOfValues {2*$Val0/2}];
      Print[ b, OnGrid {$A,$B,-t/2-0.004}{0.0001:AcoX-0.03:0.001, 0.0001:AcoY-0.001:0.001, 0} , Format Gnuplot,
        File > "bxyz0040e.txt", ChangeOfCoordinates {-$X,-$Y,$Z}, ChangeOfValues {2*$Val0/2}];

      // idem for gmsh
      Print[ b, OnGrid {$A,$B,-t/2-0.004}{0.0001:AcoX-0.03:0.002, 0.0001:AcoY-0.001:0.001, 0} ,
        File "bxyz0040e.pos" ];

      // Cut lines for gnuplot
      Print[ b, OnGrid {$A,0.0001,t/2+0.004} {0:AcoX-0.010:0.001, 0, 0} , Format Gnuplot, File "bxy0z0040i.txt" ];
      Print[ b, OnGrid {$A,0.0001,-t/2-0.004}{0:AcoX-0.010:0.001, 0, 0} , Format Gnuplot, File "bxy0z0040e.txt" ];
      Print[ b, OnGrid {0.0001,$A,t/2+0.004} {0:AcoY-0.001:0.001, 0, 0} , Format Gnuplot, File "bx0yz0040i.txt" ];
      Print[ b, OnGrid {0.0001,$A,-t/2-0.004}{0:AcoY-0.001:0.001, 0, 0} , Format Gnuplot, File "bx0yz0040e.txt" ];

      // Cut lines for gmsh
      Print[ b, OnGrid {$A,0.0001,t/2+0.004} {0:AcoX-0.010:0.001, 0, 0} , File "bxy0z0040i.pos" ];
      Print[ b, OnGrid {$A,0.0001,-t/2-0.004}{0:AcoX-0.010:0.001, 0, 0} , File "bxy0z0040e.pos" ];
      Print[ b, OnGrid {0.0001,$A,t/2+0.004} {0:AcoY-0.001:0.001, 0, 0} , File "bx0yz0040i.pos" ];
      Print[ b, OnGrid {0.0001,$A,-t/2-0.004}{0:AcoY-0.001:0.001, 0, 0} , File "bx0yz0040e.pos" ];

    }
  }
  { Name etc; NameOfPostProcessing MagSta_phi;
    Operation {
      // Other
      Print[ b, OnGrid {$A,$B,0.}{0.001:AcoX-0.03:0.001, 0.001:AcoY-0.001:0.001, 0} , Format Gnuplot,
        File "bxyz0.txt" ];
      Print[ b, OnGrid {$A,0.0001,t/2+0.004} {0:AcoX+ImaX:0.001, 0, 0} , Format Gnuplot, File "bxy0z0040i-all.txt" ];
    }
  }


}

