/* ---------------------------------------------------------------------------------------------
This is a 3D finite element calculation for a large 1/4" thick steel plate
with small semi-spherical anomaly in the middle.
The plate is magnetized with two far strong magnets. A constant zero potential
surface is defined at one side of the anomaly an another at the other side (both
have the same zero potential). There is an air volume at each side of the plate.

Symetry is exploited to calculate just one quadrant of the problem.

Janvrot IVM,  Sep 2002
-----------------------------------------------------------------------------------------------*/

// Constants
// mm = 1e-3;

Include "par.inc";  // defect parameters

// Auxiliary values
sc = Sin(Pi/4);
re = (p * p + r * r) / (2 * p);               // radius for spherical surface
AirS  = Sqrt((AcoX+ImaX)^2+(t/2+AirM)^2);     // Air above the plate at simetry plane

// Mesh size parameters (characteristic lenght)
clfd = t / 12;        // characteristic length near anomaly
clbp = 3 * t;         // characteristic length boundary
clim = t;             // characteristic length magnet

// Origin
Point(41) = {0, 0, 0, clfd};
Point(42) = {0, AcoY, 0, clbp};

// Points for the boundary

Point( 2) = {0, 0, -t/2, clfd};
Point( 3) = {0, 0, AirS, clbp};
Point( 4) = {0, 0, -AirS, clbp};

Point( 5) = {0, AcoY, t/2, clbp};
Point( 6) = {0, AcoY, -t/2, clbp};
Point( 7) = {0, AcoY, AirS, clbp};
Point( 8) = {0, AcoY, -AirS, clbp};

Point( 9) = {AcoX, AcoY, t/2, clim};
Point(10) = {AcoX, AcoY, -t/2, clim};
Point(11) = {AcoX + ImaX, AcoY, AirM, clbp};
Point(12) = {AcoX + ImaX, AcoY, -AirM, clbp};

Point( 13) = {AcoX, 0, t/2, clim};
Point( 14) = {AcoX, 0, -t/2, clim};
Point( 15) = {AcoX + ImaX, 0, AirM, clbp};
Point( 16) = {AcoX + ImaX, 0,-AirM, clbp};

// Points for the anomaly

Point(1) = {0, 0, t/2, clfd};
Point(17) = {r, 0, t/2, clfd};
Point(18) = {r*sc, r*sc, t/2, clfd};
Point(19) = {0, r, t/2, clfd};
Point(20) = {0, 0, t/2-p, clfd};
Point (21) = {0, 0, t/2+re-p, clfd}; // centro da elipse

// Points for the magnet

Point(29) = {AcoX + ImaX, AcoY, t/2, clim};
Point(30) = {AcoX + ImaX, AcoY, -t/2, clim};


Point(33) = {AcoX + ImaX, 0, t/2, clim};
Point(34) = {AcoX + ImaX, 0, -t/2, clim};

// Sufaces and volumes created with gmsh

Circle(1) = {17,1,18};
Circle(2) = {18,1,19};
Circle(3) = {19,21,20};
Circle(4) = {20,21,17};
Circle(5) = {20,21,18};
Line(6) = {6,2};
Line(7) = {2,4};
Line(8) = {4,8};
Line(9) = {8,6};
Line(10) = {6,5};
Line(11) = {5,19};
Line(12) = {5,7};
Line(13) = {7,3};
Line(14) = {3,20};
Line(15) = {20,2};
Circle(16) = {4,41,16};
Line(17) = {16,34};
Line(18) = {34,14};
Line(19) = {14,2};
Line(20) = {14,13};
Line(21) = {13,17};
Line(22) = {14,10};
Line(23) = {10,6};
Line(24) = {5,9};
Line(25) = {9,13};
Circle(26) = {12,42,8};
Line(27) = {12,16};
Line(28) = {12,30};
Line(29) = {30,34};
Line(30) = {34,33};
Line(31) = {33,29};
Line(32) = {29,9};
Line(33) = {9,10};
Line(34) = {10,30};
Line(35) = {30,29};
Line(36) = {13,33};
Line(37) = {33,15};
Line(38) = {15,11};
Circle(39) = {7,42,11};
Circle(40) = {3,41,15};
Line(41) = {11,29};
Line Loop(43) = {5,2,3};
Ruled Surface(44) = {43};
Line Loop(45) = {1,-5,4};
Ruled Surface(46) = {45};




Line Loop(47) = {11,3,15,-6,10};
Plane Surface(48) = {47};
Line Loop(49) = {24,33,23,10};
Plane Surface(50) = {49};
Line Loop(51) = {-22,20,-25,33};
Plane Surface(52) = {51};
Line Loop(53) = {-21,-20,19,-15,4};
Plane Surface(54) = {53};
Line Loop(55) = {-11,24,25,21,1,2};
Plane Surface(56) = {55};
Line Loop(57) = {-6,-23,-22,19};
Plane Surface(58) = {57};
Line Loop(59) = {-14,-13,-12,11,3};
Plane Surface(60) = {59};
Line Loop(61) = {6,7,8,9};
Plane Surface(62) = {61};
Line Loop(63) = {34,35,32,33};
Plane Surface(64) = {63};
Line Loop(65) = {-31,-30,-29,35};
Plane Surface(66) = {65};
Line Loop(67) = {36,-30,18,20};
Plane Surface(68) = {67};
Line Loop(69) = {-31,37,38,41};
Plane Surface(70) = {69};
Line Loop(71) = {-17,-27,28,29};
Plane Surface(72) = {71};
Line Loop(73) = {-32,-41,-39,-12,24};
Plane Surface(74) = {73};
Line Loop(75) = {-21,36,37,-40,14,4};
Plane Surface(76) = {75};
Line Loop(77) = {16,17,18,19,7};
Plane Surface(78) = {77};
Line Loop(79) = {-23,34,-28,26,9};
Plane Surface(80) = {79};
Line Loop(81) = {31,32,25,36};
Plane Surface(82) = {81};
Line Loop(83) = {22,34,29,18};
Plane Surface(84) = {83};
Line Loop(85) = {38,-39,13,40};
Ruled Surface(86) = {85};
Line Loop(87) = {-8,16,-27,26};
Ruled Surface(88) = {87};
Surface Loop(89) = {58,48,56,50,52,54,46,44};
Volume(90) = {89};
Surface Loop(91) = {84,52,68,82,66,64};
Volume(92) = {91};
Surface Loop(93) = {84,58,62,78,88,72,80};
Volume(94) = {93};
Surface Loop(95) = {44,46,56,60,76,82,70,86,74};
Volume(96) = {95};


// Boundary Physical Surfaces
Physical Surface(Surface0_Code) = {62,48,60,86,70,66,72,88};

// Physical Volumes
Physical Volume(Steel_Code) = {90};
Physical Volume(Magnet_Code) = {92};
Physical Volume(Air_Code) = {94,96};
